# Generated by Django 5.0.3 on 2024-03-06 00:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("jwt_drf_passwordless", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="passwordlesschallengetoken",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="passwordlesschallengetoken",
            name="short_token",
            field=models.CharField(max_length=255),
        ),
        migrations.AlterField(
            model_name="passwordlesschallengetoken",
            name="token",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="passwordlesschallengetoken",
            name="token_request_identifier",
            field=models.CharField(max_length=255),
        ),
    ]
